import java.util.*;
import javax.microedition.rms.*;

public class SettingStorageClass extends Object {
    private static final String STORAGE_NAME = "SETTING_STORAGE";

    private static final int    DECIMAL_SEPARATOR           = 1;
    private static final int    CSV_DISK                    = 2;
    private static final int    CSV_FILE_NAME               = 3;
    private static final int    GOOGLE_ACCOUNT_NAME         = 4;
    private static final int    GOOGLE_ACCOUNT_PASS         = 5;
    private static final int    GOOGLE_SPREADSHEET_NAME     = 6;
    private static final int    INITIAL_BALANCE_CATEGORY_ID = 7;
    private static final int    TRANSFER_CHARGE_CATEGORY_ID = 8;
    private static final int    TRANSFER_INCOME_CATEGORY_ID = 9;

    private static final String DEFAULT_DECIMAL_SEPARATOR           = ".";
    private static final String DEFAULT_CSV_DISK                    = "";
    private static final String DEFAULT_CSV_FILE_NAME               = "";
    private static final String DEFAULT_GOOGLE_ACCOUNT_NAME         = "";
    private static final String DEFAULT_GOOGLE_ACCOUNT_PASS         = "";
    private static final String DEFAULT_GOOGLE_SPREADSHEET_NAME     = "";
    private static final int    DEFAULT_INITIAL_BALANCE_CATEGORY_ID = 0;
    private static final int    DEFAULT_TRANSFER_CHARGE_CATEGORY_ID = 0;
    private static final int    DEFAULT_TRANSFER_INCOME_CATEGORY_ID = 0;

    private static Hashtable Hash = new Hashtable();

    public static void Initialize() throws Exception {
        int         id;
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                SettingClass setting = new SettingClass(storage.getRecord(id));

                Hash.put(new Integer(setting.GetId()), setting);
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    private static SettingClass Get(int id) {
        return (SettingClass)Hash.get(new Integer(id));
    }

    private static void Set(SettingClass setting) throws Exception {
        Exception   exception = null;
        RecordStore storage = null;

        try {
            Hash.put(new Integer(setting.GetId()), setting);

            RecordStore.deleteRecordStore(STORAGE_NAME);

            storage = RecordStore.openRecordStore(STORAGE_NAME, true);

            Enumeration hash_enum = Hash.elements();

            while (hash_enum.hasMoreElements()) {
                SettingClass item = (SettingClass)hash_enum.nextElement();

                byte byte_array[] = item.ToByteArray();

                storage.addRecord(byte_array, 0, byte_array.length);
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static String GetDecimalSeparator() {
        SettingClass setting = Get(DECIMAL_SEPARATOR);

        if (setting != null) {
            return setting.GetStringValue();
        } else {
            return DEFAULT_DECIMAL_SEPARATOR;
        }
    }

    public static void SetDecimalSeparator(String value) throws Exception {
        Set(new SettingClass(DECIMAL_SEPARATOR, false, 0, value));
    }

    public static String GetCSVDisk() {
        SettingClass setting = Get(CSV_DISK);

        if (setting != null) {
            return setting.GetStringValue();
        } else {
            return DEFAULT_CSV_DISK;
        }
    }

    public static void SetCSVDisk(String value) throws Exception {
        Set(new SettingClass(CSV_DISK, false, 0, value));
    }

    public static String GetCSVFileName() {
        SettingClass setting = Get(CSV_FILE_NAME);

        if (setting != null) {
            return setting.GetStringValue();
        } else {
            return DEFAULT_CSV_FILE_NAME;
        }
    }

    public static void SetCSVFileName(String value) throws Exception {
        Set(new SettingClass(CSV_FILE_NAME, false, 0, value));
    }

    public static String GetGoogleAccountName() {
        SettingClass setting = Get(GOOGLE_ACCOUNT_NAME);

        if (setting != null) {
            return setting.GetStringValue();
        } else {
            return DEFAULT_GOOGLE_ACCOUNT_NAME;
        }
    }

    public static void SetGoogleAccountName(String value) throws Exception {
        Set(new SettingClass(GOOGLE_ACCOUNT_NAME, false, 0, value));
    }

    public static String GetGoogleAccountPass() {
        SettingClass setting = Get(GOOGLE_ACCOUNT_PASS);

        if (setting != null) {
            return setting.GetStringValue();
        } else {
            return DEFAULT_GOOGLE_ACCOUNT_PASS;
        }
    }

    public static void SetGoogleAccountPass(String value) throws Exception {
        Set(new SettingClass(GOOGLE_ACCOUNT_PASS, false, 0, value));
    }

    public static String GetGoogleSpreadsheetName() {
        SettingClass setting = Get(GOOGLE_SPREADSHEET_NAME);

        if (setting != null) {
            return setting.GetStringValue();
        } else {
            return DEFAULT_GOOGLE_SPREADSHEET_NAME;
        }
    }

    public static void SetGoogleSpreadsheetName(String value) throws Exception {
        Set(new SettingClass(GOOGLE_SPREADSHEET_NAME, false, 0, value));
    }

    public static int GetInitialBalanceCategoryId() {
        SettingClass setting = Get(INITIAL_BALANCE_CATEGORY_ID);

        if (setting != null) {
            return setting.GetIntValue();
        } else {
            return DEFAULT_INITIAL_BALANCE_CATEGORY_ID;
        }
    }

    public static void SetInitialBalanceCategoryId(int value) throws Exception {
        Set(new SettingClass(INITIAL_BALANCE_CATEGORY_ID, false, value, ""));
    }

    public static int GetTransferChargeCategoryId() {
        SettingClass setting = Get(TRANSFER_CHARGE_CATEGORY_ID);

        if (setting != null) {
            return setting.GetIntValue();
        } else {
            return DEFAULT_TRANSFER_CHARGE_CATEGORY_ID;
        }
    }

    public static void SetTransferChargeCategoryId(int value) throws Exception {
        Set(new SettingClass(TRANSFER_CHARGE_CATEGORY_ID, false, value, ""));
    }

    public static int GetTransferIncomeCategoryId() {
        SettingClass setting = Get(TRANSFER_INCOME_CATEGORY_ID);

        if (setting != null) {
            return setting.GetIntValue();
        } else {
            return DEFAULT_TRANSFER_INCOME_CATEGORY_ID;
        }
    }

    public static void SetTransferIncomeCategoryId(int value) throws Exception {
        Set(new SettingClass(TRANSFER_INCOME_CATEGORY_ID, false, value, ""));
    }
}
