import java.util.*;
import javax.microedition.rms.*;

public class TransactionStorageClass extends Object {
    private static final String STORAGE_NAME = "TRANSACTION_STORAGE";

    private static String ValidateErrorMsg;

    private static void SortByDate(Vector vector, int left, int right) {
        if (right > left) {
            int i = left - 1;
            int j = right;

            while (true) {
                while (((TransactionClass)vector.elementAt(++i)).GetDate().getTime() < ((TransactionClass)vector.elementAt(right)).GetDate().getTime());
                while (j > 0) {
                    if (((TransactionClass)vector.elementAt(--j)).GetDate().getTime() <= ((TransactionClass)vector.elementAt(right)).GetDate().getTime()) {
                        break;
                    }
                }
                if (i >= j) {
                    break;
                }

                TransactionClass item = (TransactionClass)vector.elementAt(i);

                vector.setElementAt(vector.elementAt(j), i);
                vector.setElementAt(item, j);
            }

            TransactionClass item = (TransactionClass)vector.elementAt(i);

            vector.setElementAt(vector.elementAt(right), i);
            vector.setElementAt(item, right);

            SortByDate(vector, left,  i - 1);
            SortByDate(vector, i + 1, right);
        }
    }

    private static void SortByDateRev(Vector vector, int left, int right) {
        if (right > left) {
            int i = left - 1;
            int j = right;

            while (true) {
                while (((TransactionClass)vector.elementAt(++i)).GetDate().getTime() > ((TransactionClass)vector.elementAt(right)).GetDate().getTime());
                while (j > 0) {
                    if (((TransactionClass)vector.elementAt(--j)).GetDate().getTime() >= ((TransactionClass)vector.elementAt(right)).GetDate().getTime()) {
                        break;
                    }
                }
                if (i >= j) {
                    break;
                }

                TransactionClass item = (TransactionClass)vector.elementAt(i);

                vector.setElementAt(vector.elementAt(j), i);
                vector.setElementAt(item, j);
            }

            TransactionClass item = (TransactionClass)vector.elementAt(i);

            vector.setElementAt(vector.elementAt(right), i);
            vector.setElementAt(item, right);

            SortByDateRev(vector, left,  i - 1);
            SortByDateRev(vector, i + 1, right);
        }
    }

    private static void SortWalletTotalsPerCategoryStatisticsByTotals(Vector vector, int left, int right) {
        if (right > left) {
            int i = left - 1;
            int j = right;

            while (true) {
                while (((WalletTotalsPerCategoryStatisticsItemClass)vector.elementAt(++i)).GetTotals() < ((WalletTotalsPerCategoryStatisticsItemClass)vector.elementAt(right)).GetTotals());
                while (j > 0) {
                    if (((WalletTotalsPerCategoryStatisticsItemClass)vector.elementAt(--j)).GetTotals() <= ((WalletTotalsPerCategoryStatisticsItemClass)vector.elementAt(right)).GetTotals()) {
                        break;
                    }
                }
                if (i >= j) {
                    break;
                }

                WalletTotalsPerCategoryStatisticsItemClass item = (WalletTotalsPerCategoryStatisticsItemClass)vector.elementAt(i);

                vector.setElementAt(vector.elementAt(j), i);
                vector.setElementAt(item, j);
            }

            WalletTotalsPerCategoryStatisticsItemClass item = (WalletTotalsPerCategoryStatisticsItemClass)vector.elementAt(i);

            vector.setElementAt(vector.elementAt(right), i);
            vector.setElementAt(item, right);

            SortWalletTotalsPerCategoryStatisticsByTotals(vector, left,  i - 1);
            SortWalletTotalsPerCategoryStatisticsByTotals(vector, i + 1, right);
        }
    }

    private static void SortChargeIncomePerDayStatisticsByDayRev(Vector vector, int left, int right) {
        if (right > left) {
            int i = left - 1;
            int j = right;

            while (true) {
                while (((WalletChargeIncomePerDayStatisticsItemClass)vector.elementAt(++i)).GetDay().compareTo(((WalletChargeIncomePerDayStatisticsItemClass)vector.elementAt(right)).GetDay()) > 0);
                while (j > 0) {
                    if (((WalletChargeIncomePerDayStatisticsItemClass)vector.elementAt(--j)).GetDay().compareTo(((WalletChargeIncomePerDayStatisticsItemClass)vector.elementAt(right)).GetDay()) >= 0) {
                        break;
                    }
                }
                if (i >= j) {
                    break;
                }

                WalletChargeIncomePerDayStatisticsItemClass item = (WalletChargeIncomePerDayStatisticsItemClass)vector.elementAt(i);

                vector.setElementAt(vector.elementAt(j), i);
                vector.setElementAt(item, j);
            }

            WalletChargeIncomePerDayStatisticsItemClass item = (WalletChargeIncomePerDayStatisticsItemClass)vector.elementAt(i);

            vector.setElementAt(vector.elementAt(right), i);
            vector.setElementAt(item, right);

            SortChargeIncomePerDayStatisticsByDayRev(vector, left,  i - 1);
            SortChargeIncomePerDayStatisticsByDayRev(vector, i + 1, right);
        }
    }

    public static boolean Validate(TransactionClass transaction) {
        if (WalletStorageClass.Get(transaction.GetWalletId()) == null) {
            ValidateErrorMsg = "Invalid wallet";

            return false;
        } else if (CategoryStorageClass.Get(transaction.GetCategoryId()) == null) {
            ValidateErrorMsg = "Invalid category";

            return false;
        } else {
            return true;
        }
    }

    public static String GetValidateErrorMsg() {
        return ValidateErrorMsg;
    }

    public static TransactionClass Get(int id) throws Exception {
        TransactionClass transaction = null;
        Exception        exception = null;
        RecordStore      storage = null;

        try {
            storage     = RecordStore.openRecordStore(STORAGE_NAME, false);
            transaction = new TransactionClass(0, storage.getRecord(id));
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return transaction;
        }
    }

    public static void Add(TransactionClass transaction) throws Exception {
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage = RecordStore.openRecordStore(STORAGE_NAME, true);

            byte byte_array[] = transaction.ToByteArray();

            storage.addRecord(byte_array, 0, byte_array.length);
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static void Delete(int id) throws Exception {
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage = RecordStore.openRecordStore(STORAGE_NAME, false);
            storage.deleteRecord(id);
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static void Replace(int id, TransactionClass transaction) throws Exception {
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage = RecordStore.openRecordStore(STORAGE_NAME, false);

            byte byte_array[] = transaction.ToByteArray();

            storage.setRecord(id, byte_array, 0, byte_array.length);
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static int GetCount() throws Exception {
        int         count = 0;
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage = RecordStore.openRecordStore(STORAGE_NAME, true);
            count   = storage.getNumRecords();
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return count;
        }
    }

    public static int GetCountForMonth(Calendar month) throws Exception {
        int         id, count = 0;
        Calendar    calendar = Calendar.getInstance(month.getTimeZone());
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                TransactionClass transaction = new TransactionClass(id, storage.getRecord(id));

                calendar.setTime(transaction.GetDate());

                if (calendar.get(Calendar.MONTH) == month.get(Calendar.MONTH) &&
                    calendar.get(Calendar.YEAR)  == month.get(Calendar.YEAR)) {
                    count++;
                }
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return count;
        }
    }

    public static long GetBalanceForWalletIdMonth(int wallet_id, Calendar month) throws Exception {
        int         id;
        long        balance = 0;
        Calendar    calendar = Calendar.getInstance(month.getTimeZone());
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                TransactionClass transaction = new TransactionClass(id, storage.getRecord(id));

                calendar.setTime(transaction.GetDate());

                if (wallet_id                    == transaction.GetWalletId() &&
                    calendar.get(Calendar.MONTH) == month.get(Calendar.MONTH) &&
                    calendar.get(Calendar.YEAR)  == month.get(Calendar.YEAR)) {
                    balance = balance + transaction.GetAmount();
                }
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return balance;
        }
    }

    public static long GetBalanceForWalletIdMonthFilteredByCategoryIdDate(int wallet_id, int category_id, Calendar date, Calendar month) throws Exception {
        int         id;
        long        balance = 0;
        Calendar    calendar = Calendar.getInstance(month.getTimeZone());
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                TransactionClass transaction = new TransactionClass(id, storage.getRecord(id));

                calendar.setTime(transaction.GetDate());

                if (wallet_id                    == transaction.GetWalletId() &&
                    calendar.get(Calendar.MONTH) == month.get(Calendar.MONTH) &&
                    calendar.get(Calendar.YEAR)  == month.get(Calendar.YEAR)) {
                    if ((category_id == 0    || category_id == transaction.GetCategoryId()) &&
                        (date        == null ||
                         date.get(Calendar.DAY_OF_MONTH) == calendar.get(Calendar.DAY_OF_MONTH) &&
                         date.get(Calendar.MONTH)        == calendar.get(Calendar.MONTH) &&
                         date.get(Calendar.YEAR)         == calendar.get(Calendar.YEAR))) {
                        balance = balance + transaction.GetAmount();
                    }
                }
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return balance;
        }
    }

    public static Vector GetListForWalletIdMonthFilteredByCategoryIdDateSortedByDateRev(int wallet_id, int category_id, Calendar date, Calendar month) throws Exception {
        int         id;
        Calendar    calendar = Calendar.getInstance(month.getTimeZone());
        Vector      transactions = new Vector();
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null/*new DateRevComparator()*/, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                TransactionClass transaction = new TransactionClass(id, storage.getRecord(id));

                calendar.setTime(transaction.GetDate());

                if (wallet_id                    == transaction.GetWalletId() &&
                    calendar.get(Calendar.MONTH) == month.get(Calendar.MONTH) &&
                    calendar.get(Calendar.YEAR)  == month.get(Calendar.YEAR)) {
                    if ((category_id == 0    || category_id == transaction.GetCategoryId()) &&
                        (date        == null ||
                         date.get(Calendar.DAY_OF_MONTH) == calendar.get(Calendar.DAY_OF_MONTH) &&
                         date.get(Calendar.MONTH)        == calendar.get(Calendar.MONTH) &&
                         date.get(Calendar.YEAR)         == calendar.get(Calendar.YEAR))) {
                        transactions.addElement(transaction);
                    }
                }
            }

            SortByDateRev(transactions, 0, transactions.size() - 1);
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return transactions;
        }
    }

    public static Vector GetListFilteredByWalletIdMonthSortedByDate(int wallet_id, Calendar month) throws Exception {
        int         id;
        Calendar    calendar = Calendar.getInstance();
        Vector      transactions = new Vector();
        Exception   exception = null;
        RecordStore storage = null;

        if (month != null) {
            calendar.setTimeZone(month.getTimeZone());
        }

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                TransactionClass transaction = new TransactionClass(id, storage.getRecord(id));

                calendar.setTime(transaction.GetDate());

                if ((wallet_id == 0    || wallet_id                    == transaction.GetWalletId()) &&
                    (month     == null || calendar.get(Calendar.MONTH) == month.get(Calendar.MONTH) &&
                                          calendar.get(Calendar.YEAR)  == month.get(Calendar.YEAR))) {
                    transactions.addElement(transaction);
                }
            }

            SortByDate(transactions, 0, transactions.size() - 1);
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return transactions;
        }
    }

    public static Vector GetTotalsPerCategoryStatisticsForWalletIdMonthSortedByAmount(int wallet_id, Calendar month) throws Exception {
        int         id;
        Calendar    calendar = Calendar.getInstance(month.getTimeZone());
        Hashtable   category_amounts = new Hashtable();
        Vector      statistics = new Vector();
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                TransactionClass transaction = new TransactionClass(id, storage.getRecord(id));

                calendar.setTime(transaction.GetDate());

                if (wallet_id                    == transaction.GetWalletId() &&
                    calendar.get(Calendar.MONTH) == month.get(Calendar.MONTH) &&
                    calendar.get(Calendar.YEAR)  == month.get(Calendar.YEAR)) {
                    CategoryClass category = CategoryStorageClass.Get(transaction.GetCategoryId());

                    if (category != null) {
                        String category_name = category.GetName();

                        if (category_amounts.containsKey(category_name)) {
                            WalletTotalsPerCategoryStatisticsItemClass statistics_item = ((WalletTotalsPerCategoryStatisticsItemClass)category_amounts.get(category_name));

                            statistics_item.SetTotals(statistics_item.GetTotals() + transaction.GetAmount());

                            category_amounts.put(category_name, statistics_item);
                        } else {
                            category_amounts.put(category_name, new WalletTotalsPerCategoryStatisticsItemClass(category_name, transaction.GetAmount()));
                        }
                    } else {
                        throw (new Exception("Invalid category"));
                    }
                }
            }

            for (Enumeration hash_enum = category_amounts.keys(); hash_enum.hasMoreElements();) {
                statistics.addElement(category_amounts.get(hash_enum.nextElement()));
            }

            SortWalletTotalsPerCategoryStatisticsByTotals(statistics, 0, statistics.size() - 1);
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return statistics;
        }
    }

    public static Vector GetChargeIncomePerDayStatisticsForWalletIdMonthSortedByDayRev(int wallet_id, Calendar month) throws Exception {
        int                                         id;
        Calendar                                    calendar = Calendar.getInstance(month.getTimeZone());
        Hashtable                                   day_charges_incomes = new Hashtable();
        Vector                                      statistics = new Vector();
        Exception                                   exception = null;
        RecordStore                                 storage = null;
        WalletChargeIncomePerDayStatisticsItemClass total_item = new WalletChargeIncomePerDayStatisticsItemClass("Total", 0, 0);

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                TransactionClass transaction = new TransactionClass(id, storage.getRecord(id));

                calendar.setTime(transaction.GetDate());

                if (wallet_id                    == transaction.GetWalletId() &&
                    calendar.get(Calendar.MONTH) == month.get(Calendar.MONTH) &&
                    calendar.get(Calendar.YEAR)  == month.get(Calendar.YEAR)) {
                    String day = UtilClass.ZeroPad(calendar.get(Calendar.DAY_OF_MONTH)) + " " +
                                 UtilClass.GetCalendarMonthName(calendar);

                    if (transaction.GetAmount() < 0) {
                        if (day_charges_incomes.containsKey(day)) {
                            WalletChargeIncomePerDayStatisticsItemClass statistics_item = ((WalletChargeIncomePerDayStatisticsItemClass)day_charges_incomes.get(day));

                            statistics_item.SetCharge(statistics_item.GetCharge() + transaction.GetAmount());

                            day_charges_incomes.put(day, statistics_item);
                        } else {
                            day_charges_incomes.put(day, new WalletChargeIncomePerDayStatisticsItemClass(day, transaction.GetAmount(), 0));
                        }

                        total_item.SetCharge(total_item.GetCharge() + transaction.GetAmount());
                    } else if (transaction.GetAmount() > 0) {
                        if (day_charges_incomes.containsKey(day)) {
                            WalletChargeIncomePerDayStatisticsItemClass statistics_item = ((WalletChargeIncomePerDayStatisticsItemClass)day_charges_incomes.get(day));

                            statistics_item.SetIncome(statistics_item.GetIncome() + transaction.GetAmount());

                            day_charges_incomes.put(day, statistics_item);
                        } else {
                            day_charges_incomes.put(day, new WalletChargeIncomePerDayStatisticsItemClass(day, 0, transaction.GetAmount()));
                        }

                        total_item.SetIncome(total_item.GetIncome() + transaction.GetAmount());
                    }
                }
            }

            for (Enumeration hash_enum = day_charges_incomes.keys(); hash_enum.hasMoreElements();) {
                statistics.addElement(day_charges_incomes.get(hash_enum.nextElement()));
            }

            statistics.addElement(total_item);

            SortChargeIncomePerDayStatisticsByDayRev(statistics, 0, statistics.size() - 1);
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return statistics;
        }
    }

    public static void DeleteForMonth(Calendar month) throws Exception {
        int         id;
        Calendar    calendar = Calendar.getInstance(month.getTimeZone());
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                TransactionClass transaction = new TransactionClass(id, storage.getRecord(id));

                calendar.setTime(transaction.GetDate());

                if (calendar.get(Calendar.MONTH) == month.get(Calendar.MONTH) &&
                    calendar.get(Calendar.YEAR)  == month.get(Calendar.YEAR)) {
                    storage.deleteRecord(id);
                }
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw(exception);
        }
    }

    public static void DeleteForWalletId(int wallet_id) throws Exception {
        int         id;
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                TransactionClass transaction = new TransactionClass(id, storage.getRecord(id));

                if (transaction.GetWalletId() == wallet_id) {
                    storage.deleteRecord(id);
                }
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw(exception);
        }
    }

    public static void DeleteForCategoryId(int category_id) throws Exception {
        int         id;
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                TransactionClass transaction = new TransactionClass(id, storage.getRecord(id));

                if (transaction.GetCategoryId() == category_id) {
                    storage.deleteRecord(id);
                }
            }
        } catch (Exception ex) {
            // Ignore
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw(exception);
        }
    }
}
