import java.io.*;
import java.util.*;
import javax.microedition.io.file.*;

public class UtilClass extends Object {
    private static final char   HEX[]          = "0123456789ABCDEF".toCharArray();
    private static final String URL_UNRESERVED = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_.~";

    public static String URLEncode(String str) {
        byte                  bytes[] = new byte[0];
        ByteArrayOutputStream stream = null;
        DataOutputStream      data_stream = null;
        StringBuffer          buf = new StringBuffer();

        try {
            stream      = new ByteArrayOutputStream();
            data_stream = new DataOutputStream(stream);

            data_stream.writeUTF(str);

            bytes = stream.toByteArray();
        } catch (Exception ex) {
            // Ignore
        } finally {
            if (data_stream != null) {
                try {
                    data_stream.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (stream != null) {
                try {
                    stream.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }

        for (int i = 2; i < bytes.length; i++) {
            if (URL_UNRESERVED.indexOf(bytes[i]) >= 0) {
                buf.append((char)bytes[i]);
            } else {
                buf.append('%').append(HEX[(bytes[i] >> 4) & 0x0f]).append(HEX[bytes[i] & 0x0f]);
            }
        }

        return buf.toString();
    }

    public static String MoneyToString(long money) {
        String result;

        if (money < 0) {
            result = "-" + String.valueOf(Math.abs(money / 100));
        } else {
            result =       String.valueOf(Math.abs(money / 100));
        }

        if (Math.abs(money % 100) == 0) {
            result = result + SettingStorageClass.GetDecimalSeparator() + "00";
        } else if (Math.abs(money % 100) < 10) {
            result = result + SettingStorageClass.GetDecimalSeparator() + "0" + String.valueOf(Math.abs(money % 100));
        } else {
            result = result + SettingStorageClass.GetDecimalSeparator() + String.valueOf(Math.abs(money % 100));
        }

        return result;
    }

    public static long StringToMoney(String str) {
        String int_str, frac_str;

        if (str.indexOf('.') != -1) {
            int delim_pos = str.indexOf('.');

            int_str  = str.substring(0, delim_pos);
            frac_str = str.substring(delim_pos + 1, str.length());
        } else if (str.indexOf(',') != -1) {
            int delim_pos = str.indexOf(',');

            int_str  = str.substring(0, delim_pos);
            frac_str = str.substring(delim_pos + 1, str.length());
        } else {
            int_str  = str;
            frac_str = "0";
        }

        try {
            if (!int_str.equals("") && !frac_str.equals("") && Character.isDigit(frac_str.charAt(0))) {
                long result = Math.abs(Long.parseLong(int_str) * 100);

                if (frac_str.length() == 1) {
                    if (int_str.charAt(0) == '-') {
                        result = 0 - result - Long.parseLong(frac_str) * 10;
                    } else {
                        result =     result + Long.parseLong(frac_str) * 10;
                    }
                } else {
                    if (int_str.charAt(0) == '-') {
                        result = 0 - result - Long.parseLong(frac_str.substring(0, 2));
                    } else {
                        result =     result + Long.parseLong(frac_str.substring(0, 2));
                    }
                }

                return result;
            } else {
                throw (new NumberFormatException(str));
            }
        } catch (NumberFormatException ex) {
            throw (new NumberFormatException(str));
        }
    }

    public static Vector SplitToLines(String str) {
        int    curr_pos = 0, next_pos;
        Vector result = new Vector();

        while (curr_pos < str.length()) {
            next_pos = str.indexOf("\r\n", curr_pos);

            if (next_pos != -1) {
                result.addElement(str.substring(curr_pos, next_pos));

                curr_pos = next_pos + 2;
            } else {
                next_pos = str.indexOf("\n", curr_pos);

                if (next_pos != -1) {
                    result.addElement(str.substring(curr_pos, next_pos));

                    curr_pos = next_pos + 1;
                } else {
                    result.addElement(str.substring(curr_pos, str.length()));

                    curr_pos = str.length();
                }
            }
        }

        return result;
    }

    public static String ZeroPad(int num) {
        if (num < 10) {
            return new String("0") + String.valueOf(num);
        } else {
            return                   String.valueOf(num);
        }
    }

    public static String GetCalendarMonthName(Calendar calendar) {
        switch (calendar.get(Calendar.MONTH)) {
            case Calendar.JANUARY:   return "Jan";
            case Calendar.FEBRUARY:  return "Feb";
            case Calendar.MARCH:     return "Mar";
            case Calendar.APRIL:     return "Apr";
            case Calendar.MAY:       return "May";
            case Calendar.JUNE:      return "Jun";
            case Calendar.JULY:      return "Jul";
            case Calendar.AUGUST:    return "Aug";
            case Calendar.SEPTEMBER: return "Sep";
            case Calendar.OCTOBER:   return "Oct";
            case Calendar.NOVEMBER:  return "Nov";
            case Calendar.DECEMBER:  return "Dec";
            default:                 return "???";
        }
    }

    public static int GetCalendarMonthId(String name) {
        if (name.equals("Jan")) {
            return Calendar.JANUARY;
        } else if (name.equals("Feb")) {
            return Calendar.FEBRUARY;
        } else if (name.equals("Mar")) {
            return Calendar.MARCH;
        } else if (name.equals("Apr")) {
            return Calendar.APRIL;
        } else if (name.equals("May")) {
            return Calendar.MAY;
        } else if (name.equals("Jun")) {
            return Calendar.JUNE;
        } else if (name.equals("Jul")) {
            return Calendar.JULY;
        } else if (name.equals("Aug")) {
            return Calendar.AUGUST;
        } else if (name.equals("Sep")) {
            return Calendar.SEPTEMBER;
        } else if (name.equals("Oct")) {
            return Calendar.OCTOBER;
        } else if (name.equals("Nov")) {
            return Calendar.NOVEMBER;
        } else if (name.equals("Dec")) {
            return Calendar.DECEMBER;
        } else {
            return -1;
        }
    }

    public static Calendar GetPrevMonth(Calendar month) {
        int      curr_month = month.get(Calendar.MONTH);
        Calendar prev_month = Calendar.getInstance(month.getTimeZone());

        prev_month.setTime(month.getTime());

        if (curr_month == Calendar.JANUARY) {
            prev_month.set(Calendar.YEAR, prev_month.get(Calendar.YEAR) - 1);
            prev_month.set(Calendar.MONTH, Calendar.DECEMBER);
        } else {
            prev_month.set(Calendar.MONTH, curr_month - 1);
        }

        return prev_month;
    }

    public static String FormatDate(Date date) {
        Calendar calendar = Calendar.getInstance();

        calendar.setTime(date);

        return ZeroPad(calendar.get(Calendar.DAY_OF_MONTH)) + " " + GetCalendarMonthName(calendar) + " " +
               String.valueOf(calendar.get(Calendar.YEAR)) + " " + ZeroPad(calendar.get(Calendar.HOUR_OF_DAY)) + ":" +
               ZeroPad(calendar.get(Calendar.MINUTE)) + ":" + ZeroPad(calendar.get(Calendar.SECOND)) + " " +
               calendar.getTimeZone().getID();
    }

    public static Date ParseDate(String str) throws Exception {
        int      curr_pos = 0, next_pos;
        Calendar calendar = Calendar.getInstance();

        next_pos = str.indexOf(" ", curr_pos);
        if (next_pos == -1) {
            throw (new Exception("Invalid date format"));
        } else {
            calendar.set(Calendar.DAY_OF_MONTH, Integer.parseInt(str.substring(curr_pos, next_pos)));

            curr_pos = next_pos + 1;
        }

        next_pos = str.indexOf(" ", curr_pos);
        if (next_pos == -1) {
            throw (new Exception("Invalid date format"));
        } else {
            calendar.set(Calendar.MONTH, GetCalendarMonthId(str.substring(curr_pos, next_pos)));

            curr_pos = next_pos + 1;
        }

        next_pos = str.indexOf(" ", curr_pos);
        if (next_pos == -1) {
            throw (new Exception("Invalid date format"));
        } else {
            calendar.set(Calendar.YEAR, Integer.parseInt(str.substring(curr_pos, next_pos)));

            curr_pos = next_pos + 1;
        }

        next_pos = str.indexOf(":", curr_pos);
        if (next_pos == -1) {
            throw (new Exception("Invalid date format"));
        } else {
            calendar.set(Calendar.HOUR_OF_DAY, Integer.parseInt(str.substring(curr_pos, next_pos)));

            curr_pos = next_pos + 1;
        }

        next_pos = str.indexOf(":", curr_pos);
        if (next_pos == -1) {
            throw (new Exception("Invalid date format"));
        } else {
            calendar.set(Calendar.MINUTE, Integer.parseInt(str.substring(curr_pos, next_pos)));

            curr_pos = next_pos + 1;
        }

        next_pos = str.indexOf(" ", curr_pos);
        if (next_pos == -1) {
            throw (new Exception("Invalid date format"));
        } else {
            calendar.set(Calendar.SECOND, Integer.parseInt(str.substring(curr_pos, next_pos)));

            curr_pos = next_pos + 1;
        }

        if (curr_pos < str.length()) {
            calendar.setTimeZone(TimeZone.getTimeZone(str.substring(curr_pos, str.length())));
        }

        return calendar.getTime();
    }

    public static boolean ValidateFileConnectionAPI() {
        if (System.getProperty("microedition.io.file.FileConnection.version") != null) {
            return true;
        } else {
            return false;
        }
    }

    public static Vector GetFileSystemRoots() {
        Vector roots = new Vector();

        try {
            Enumeration root_enum = FileSystemRegistry.listRoots();

            while (root_enum.hasMoreElements()) {
                roots.addElement(root_enum.nextElement());
            }
        } catch (Exception ex) {
            // Ignore
        }

        return roots;
    }
}
