import java.util.*;

public class WalletChargeIncomePerDayStatisticsTableClass extends CalendarTableClass {
    private static final String TITLE[]       = {"Day", "Charge", "Income"};
    private static final int    ALIGN[]       = {CalendarTableClass.ITEM_ALIGN_LEFT,
                                                 CalendarTableClass.ITEM_ALIGN_RIGHT,
                                                 CalendarTableClass.ITEM_ALIGN_RIGHT};
    private static final int    FLOAT_COL_NUM = 0;

    private int WalletId;

    public WalletChargeIncomePerDayStatisticsTableClass(Calendar month, int wallet_id, String wallet_name) {
        super(wallet_name, "", month, TITLE, ALIGN, FLOAT_COL_NUM);

        WalletId = wallet_id;

        RefreshData();
    }

    protected void RefreshData() {
        Vector statistics;

        try {
            SetLabel(UtilClass.GetCalendarMonthName(Month) + " " + String.valueOf(Month.get(Calendar.YEAR)) + " : " +
                     UtilClass.MoneyToString(TransactionStorageClass.GetBalanceForWalletIdMonth(WalletId, Month)));
        } catch (Exception ex) {
            SetLabel(UtilClass.GetCalendarMonthName(Month) + " " + String.valueOf(Month.get(Calendar.YEAR)));
        }

        try {
            statistics = TransactionStorageClass.GetChargeIncomePerDayStatisticsForWalletIdMonthSortedByDayRev(WalletId, Month);

            Data        = new String[statistics.size()][GetCols()];
            Rows        = Data.length;
            SelectedRow = 0;

            for (int i = 0; i < statistics.size(); i++) {
                WalletChargeIncomePerDayStatisticsItemClass statistics_item  = (WalletChargeIncomePerDayStatisticsItemClass)statistics.elementAt(i);

                Data[i][0] = statistics_item.GetDay();

                if (statistics_item.GetCharge() > 0) {
                    Data[i][1] = "+" + UtilClass.MoneyToString(statistics_item.GetCharge());
                } else {
                    Data[i][1] =       UtilClass.MoneyToString(statistics_item.GetCharge());
                }
                if (statistics_item.GetIncome() > 0) {
                    Data[i][2] = "+" + UtilClass.MoneyToString(statistics_item.GetIncome());
                } else {
                    Data[i][2] =       UtilClass.MoneyToString(statistics_item.GetIncome());
                }
            }
        } catch (Exception ex) {
            Data        = new String[0][GetCols()];
            Rows        = Data.length;
            SelectedRow = 0;
        }
    }
}
